/*----------------------------------------------------------------------------
 *      RL-ARM - RTX
 *----------------------------------------------------------------------------
 *      Name:    BLINKY.C
 *      Purpose: RTX example program
 *----------------------------------------------------------------------------
 *      This code is part of the RealView Run-Time Library.
 *      Copyright (c) 2006-2013 KEIL - An ARM Company. All rights reserved.
 *---------------------------------------------------------------------------*/
#include "cmsis_os.h"                   // ARM::CMSIS:RTOS:Keil RTX
#include "MKL02Z4.h"                    // Device header


osThreadId tid_phaseA;                  /* Thread id of task: phase_a */
osThreadId tid_phaseB;                  /* Thread id of task: phase_b */
osThreadId tid_phaseC;                  /* Thread id of task: phase_c */
osThreadId tid_phaseD;                  /* Thread id of task: phase_d */
osThreadId tid_clock;                   /* Thread id of task: clock   */

#define LED_NUM     3                   /* Number of user LEDs                */
const uint32_t led_mask[] = {1UL << 6, 1UL << 7, 1UL << 10};

#define LED_RED    0
#define LED_GREEN  1
#define LED_BLUE   2
#define LED_CLK    4

/*----------------------------------------------------------------------------
  Function that initializes LEDs
 *----------------------------------------------------------------------------*/
void LED_Initialize(void) {

  SIM->SCGC5    |= (1UL <<  10);                     /* Enable Clock to Port B */ 
  PORTB->PCR[6]  = (1UL <<  8);                      /* Pin PTB6  is GPIO */
  PORTB->PCR[7]  = (1UL <<  8);                      /* Pin PTB7  is GPIO */
  PORTB->PCR[10] = (1UL <<  8);                      /* Pin PTB10 is GPIO */

  FPTB->PDOR = (led_mask[0] | led_mask[1] |\
               led_mask[2] );          /* switch Red/Green/Blue LED off  */
  FPTB->PDDR = (led_mask[0] | led_mask[1] |\
               led_mask[2] );          /* enable PTB6/7/10 as Output */


}

/*----------------------------------------------------------------------------
  Function that turns on Red LED
 *----------------------------------------------------------------------------*/
void LEDRed_On (void) {
  FPTB->PSOR   = led_mask[LED_BLUE];   /* Blue LED Off*/
  FPTB->PSOR   = led_mask[LED_GREEN];  /* Green LED Off*/
  FPTB->PCOR   = led_mask[LED_RED];    /* Red LED On*/
}

/*----------------------------------------------------------------------------
  Function that turns on Blue LED
 *----------------------------------------------------------------------------*/
void LEDGreen_On (void) {
  FPTB->PSOR   = led_mask[LED_BLUE];   /* Blue LED Off*/
  FPTB->PSOR   = led_mask[LED_RED];    /* Red LED Off */
  FPTB->PCOR   = led_mask[LED_GREEN];  /* Green LED On*/
}

/*----------------------------------------------------------------------------
  Function that turns on Green LED
 *----------------------------------------------------------------------------*/
void LEDBlue_On (void) {
  FPTB->PSOR   = led_mask[LED_RED];    /* Red LED Off*/
  FPTB->PSOR   = led_mask[LED_GREEN];  /* Green LED Off*/
  FPTB->PCOR   = led_mask[LED_BLUE];   /* Blue LED On*/
}


/*----------------------------------------------------------------------------
 *      Function 'signal_func' called from multiple threads
 *---------------------------------------------------------------------------*/
void signal_func (osThreadId tid)  {
  osSignalSet(tid_clock, 0x0100);           /* set signal to clock thread    */
  osDelay(50);                              /* delay 50ms                     */
  osSignalSet(tid_clock, 0x0100);           /* set signal to clock thread    */
  osDelay(50);                              /* delay 50ms                     */
  osSignalSet(tid, 0x0001);                 /* set signal to thread 'thread' */
  osDelay(50);                              /* delay 50ms                     */
}

/*----------------------------------------------------------------------------
 *        Thread 1 'phaseA': Phase A output
 *---------------------------------------------------------------------------*/
void phaseA (void const *argument) {
  for (;;) {
    osSignalWait(0x0001, osWaitForever);    /* wait for an event flag 0x0001 */
    LEDRed_On();
    osDelay(1000);                          /* delay 1000ms                  */
    signal_func(tid_phaseB);                /* call common signal function   */
  }
}

/*----------------------------------------------------------------------------
 *        Thread 2 'phaseB': Phase B output
 *---------------------------------------------------------------------------*/
void phaseB (void const *argument) {
  for (;;) {
    osSignalWait(0x0001, osWaitForever);    /* wait for an event flag 0x0001 */
    LEDBlue_On();
    osDelay(1000);                          /* delay 1000ms                  */
    signal_func(tid_phaseC);                /* call common signal function   */
  }
}

/*----------------------------------------------------------------------------
 *        Thread 3 'phaseC': Phase C output
 *---------------------------------------------------------------------------*/
void phaseC (void const *argument) {
  for (;;) {
    osSignalWait(0x0001, osWaitForever);    /* wait for an event flag 0x0001 */
    LEDRed_On();
    osDelay(1000);                          /* delay 1000ms                  */
    signal_func(tid_phaseD);                /* call common signal function   */
  }
}

/*----------------------------------------------------------------------------
 *        Thread 4 'phaseD': Phase D output
 *---------------------------------------------------------------------------*/
void phaseD (void const *argument) {
  for (;;) {
    osSignalWait(0x0001, osWaitForever);    /* wait for an event flag 0x0001 */
    LEDBlue_On();
    osDelay(1000);                          /* delay 1000ms                  */
    signal_func(tid_phaseA);                /* call common signal function   */
  }
}

/*----------------------------------------------------------------------------
 *        Thread 5 'clock': Signal Clock
 *---------------------------------------------------------------------------*/
void clock (void const *argument) {
  for (;;) {
    osSignalWait(0x0100, osWaitForever);    /* wait for an event flag 0x0001 */
    LEDGreen_On();
    osDelay(200);                           /* delay  200ms                  */
  }
}


osThreadDef(phaseA, osPriorityNormal, 1, 0);
osThreadDef(phaseB, osPriorityNormal, 1, 0);
osThreadDef(phaseC, osPriorityNormal, 1, 0);
osThreadDef(phaseD, osPriorityNormal, 1, 0);
osThreadDef(clock,  osPriorityNormal, 1, 0);

/*----------------------------------------------------------------------------
 *      Main: Initialize and start RTX Kernel
 *---------------------------------------------------------------------------*/
int main (void) {

  SystemCoreClockUpdate();
  LED_Initialize();                         /* Initialize the LEDs           */

  tid_phaseA = osThreadCreate(osThread(phaseA), NULL);
  tid_phaseB = osThreadCreate(osThread(phaseB), NULL);
  tid_phaseC = osThreadCreate(osThread(phaseC), NULL);
  tid_phaseD = osThreadCreate(osThread(phaseD), NULL);
  tid_clock  = osThreadCreate(osThread(clock),  NULL);

  osSignalSet(tid_phaseA, 0x0001);          /* set signal to phaseA thread   */

  osDelay(osWaitForever);
  while(1);
}
